import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:ocg_app/ocg_app.dart';
import 'package:http/http.dart' as http;

import '../models/user.dart';

abstract class UserRepository {
  Future<AppResponse<User>> getSelf();
  Future<AppResponse<String>> getSelfId();
  Future<AppResponse<bool>> updateSelf(User user);
  Future<AppResponse<bool>> blockUser(String userId, bool block);
  Future<AppResponse<User>> getUser(String userId);
  Future<AppResponse<List<User>>> getUsersList(List<String> userIds);

  Future<AppResponse<List<User>>> getAllUsers();
}

class UserTestRepository extends UserRepository {
  static Map<String, User> _users = {};
  static String _currentId;

  @override
  Future<AppResponse<List<User>>> getAllUsers() async {
    return AppResponse.named(data: _users.values.toList());
  }

  @override
  Future<AppResponse<User>> getUser(String userId) async {
    return AppResponse.named(data: _users[userId]);
  }

  @override
  Future<AppResponse<User>> getSelf() async {
    return AppResponse.named(data: _users[_currentId]);
  }

  @override
  Future<AppResponse<String>> getSelfId() async {
    return AppResponse.named(data: _currentId);
  }

  @override
  Future<AppResponse<bool>> blockUser(String userId, bool block) {
    // TODO: implement blockUser
    throw UnimplementedError();

    @override
    Future<AppResponse<bool>> updateSelf(User user) {
      // TODO: implement updateSelf
      throw UnimplementedError();
    }

    @override
    Future<AppResponse<List<User>>> getUsersList(List<String> userIds) {
      // TODO: implement getUsersList
      throw UnimplementedError();
    }
  }

  @override
  Future<AppResponse<List<User>>> getUsersList(List<String> userIds) {
    // TODO: implement getUsersList
    throw UnimplementedError();
  }

  @override
  Future<AppResponse<bool>> updateSelf(User user) {
    // TODO: implement updateSelf
    throw UnimplementedError();
  }
}

class UserNetworkRepository extends UserRepository {
  final TokenRepository tokenRepository;
  final http.BaseClient client;

  UserNetworkRepository(this.tokenRepository, this.client);

  @override
  Future<AppResponse<List<User>>> getAllUsers() async {
    try {
      String url = "/contacts";
      var response = await client.get(
        url,
      );
      var json = jsonDecode(response.body);
      List<String> contacts = List<String>.from(json["contacts"] as List);
      Set<User> users = Set<User>();
      await Future.wait(contacts.map((val) async {
        AppResponse<User> userResponse = await getUser(val);
        if (userResponse.isSuccess) users.add(userResponse.data);
      }));
      return AppResponse.named(data: users.toList());
    } catch (e) {
      return AppResponse.named(error: e.toString());
    }
  }

  @override
  Future<AppResponse<User>> getSelf() async {
    try {
      String url = "/self";
      var response = await client.get(url);
      var json = jsonDecode(response.body);
      return AppResponse.named(data: User.fromJson(json["profile"]));
    } catch (e) {
      debugPrint("Error $e");
      return AppResponse.named(error: e.toString());
    }
  }

  @override
  Future<AppResponse<String>> getSelfId() async {
    AppResponse<AppToken> response = await tokenRepository.getToken();
    if (response.isSuccess) {
      return AppResponse.named(data: response.data.userId);
    }
    return AppResponse.named(error: response.error);
  }

  @override
  Future<AppResponse<User>> getUser(String userId) async {
    try {
      String url = "/users/$userId/";
      var response = await client.get(url);
      var json = jsonDecode(response.body);
      return AppResponse.named(data: User.fromJson(json["profile"]));
    } catch (e) {
      return AppResponse.named(error: e.toString());
    }
  }

  @override
  Future<AppResponse<bool>> updateSelf(User user) async {
    try {
      String url = "/users/${user.id}/";
      await client.post(url, body: jsonEncode(user.toJson()));
      return AppResponse.named(data: true);
    } catch (e) {
      return AppResponse.named(error: e.toString());
    }
  }

  @override
  Future<AppResponse<bool>> blockUser(String userId, bool block) async {
    try {
      String url = "/users/$userId/${block ? "block" : "unblock"}";
      var response = await client.post(
        url,
      );
      return AppResponse.named(data: response.statusCode == 200);
    } catch (e) {
      debugPrint(
        "Error $e",
      );
      return AppResponse.named(error: e.toString());
    }
  }

  @override
  Future<AppResponse<List<User>>> getUsersList(List<String> userIds) async {
    try {
      List<User> users = [];
      users = await Future.wait(userIds.map((e) async {
        AppResponse<User> userResponse = await getUser(e);
        if (userResponse.isSuccess) {
          return userResponse.data;
        }
      }).toList());
      users = users.where((element) => element != null).toList();
      return AppResponse.named(data: users);
    } catch (e) {
      return AppResponse.named(error: e.toString());
    }
  }
}
